/* recons - Reconstruccion de datos comprimidos mediante red de Sanger  */
/* +------------------------------------------------------------------+ */
/* | Descripción:                                                     | */
/* | A partir del los pesos de sanger y los datos comprimidos         | */
/* | reconstruye las entradas originales utilizando un número         | */
/* | determinado de componentes.                                      | */
/* |                                                                  | */
/* | Dependencias:                                                    | */
/* | Hace uso de hermat (libreria de funciones para la lectura y      | */
/* | escritura de matrices de floats).                                | */
/* |                                                                  | */
/* | Problemas:                                                       | */
/* |                                                                  | */
/* | ToDo:                                                            | */
/* | Emitir información de varianza representada                      | */
/* | Emitir información, si se da el original, de error de reconstr.  | */
/* | Utilizar genopts para entrada de parámetros por línea y reestr.  | */
/* | Utilizar el estilo de estilo.c                                   | */
/* |                                                                  | */
/* | Autor: Patricio Garcia Baez - pgarcia@ull.es                     | */
/* +------------------------------------------------------------------+ */
/*                                                                      */
/*   versión:  2.0                                                      */
/*   modificación: 18/12/2003 Mas comentarios, ayuda                    */
/*                                                                      */
/*                                                                      */
/************************************************************************/

#include "hermat.h"
#include <stdlib.h>

char     *program;

void help()
{
    fprintf(stderr, "Usar: %s fich_ws fich_red n c [c_real]\n", program);
    fprintf(stderr, "Reconstruye los datos comprimidos mediante red de Sanger hacia la salida estandar.\n");
    fprintf(stderr, "Opciones:\n");
    fprintf(stderr, " fich_in_ws => Nombre del ficheros de pesos de Sanger (n*c)\n");
    fprintf(stderr, " fich_red  => Nombre del ficheros de datos comprimidos (m*c)\n");
    fprintf(stderr, " n  => Número de caracteristicas originales a regenerar\n");
    fprintf(stderr, " cp => Número de caracteristicas extraidas por Sanger\n");
    fprintf(stderr, " cpreal  => Número de caracteristicas reales a emplear\n");
    exit(1);
}


int main(int argc, char *argv[]) 
{
#define FL 80
    unsigned i, j, leidas, m, n, n_real;
    vector *ptvw;
    register elem *ptew, *ptem;

    char fichin[FL];
    FILE *fi1, *fi2;

    matriz w;
    vector mat;

    /* Leer argumentos desde la línea de comandos */

    program = argv[0];

    if ((argc < 5) || (argc > 6))
	help();

    sscanf(argv[3],"%d",&m);
    sscanf(argv[4],"%d",&n);
    n_real = n;
    if (argc > 5)
	sscanf(argv[5],"%d",&n_real);

    sscanf(argv[1],"%s",fichin);

    w = reserva_matriz(m, n);
    if ((fi1 = fopen(fichin, "r")) == NULL)
	Err("No se pudo abrir fichero de pesos de Sanger");
    lee_matriz(fi1, m, n, w);
    fclose(fi1);

    sscanf(argv[2],"%s",fichin);
    if ((fi2 = fopen(fichin, "r")) == NULL)
	Err("No se pudo abrir fichero de datos comprimidos");

    mat = reserva_vector(n);  
    leidas = lee_matriz(fi2, 1, n, &mat);

    while (leidas == 1) {
      
	for (j = m, ptvw = w; j--; ptvw++) {
	    register elem   sum = 0;
	    for (i = n_real, ptem = mat, ptew = *ptvw; i--;) {
		sum += (*ptew++) * (*ptem++);
	    }
	    printf("%f%c",sum, (j==0)?'\n':'\t');
	}
	leidas = lee_matriz(fi2, 1, n, &mat);
    }

    fclose(fi2);
    libera_vector(mat);
    libera_matriz(m, w);

    return 0;
}
